import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { CareerPageCmsRepository } from "./career-page-cms.repository";
import { UpdateCareerPageCmsDto } from "./dto/update-career-page-cms.dto";
import { CareerPageCms } from "./entities/career-page-cms.entity";
import { UploadedMulterFileI } from "../../SpacesModule/SpacesService";
import { DoSpacesService } from "../../SpacesModule/SpacesService/doSpacesService";

@Injectable()
export class CareerPageCmsService {
    constructor(
      @InjectRepository(CareerPageCmsRepository)
      private readonly repository: CareerPageCmsRepository,
      private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
      dto: UpdateCareerPageCmsDto,
      files: {
          heroSectionBgImage: UploadedMulterFileI
      }
    ): Promise<CareerPageCms | Error> {
        if (files && files.heroSectionBgImage) {
            const heroSectionBgImage: any = await this.doSpaceService.uploadFile(files.heroSectionBgImage[0], "CareerPageCms/");
            dto.heroSectionBgImage = heroSectionBgImage;
        }
        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                  Constants.NOT_FOUND,
                  HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                  Constants.UPDATE_FAILED,
                  HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<CareerPageCms[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}