import {
    Body,
    Controller,
    Get,
    HttpCode,
    HttpStatus,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { Constants } from "src/utils/constants";
import { LandingPageCmsService } from "./landing-page-cms.service";
import { UpdateLandingPageCmsDto } from "./dto/update-landing-page-cms.dto";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";

@Controller({ path: "landing-page-cms", version: Constants.API_VERSION_1 })

export class LandingPageCmsController {
    constructor(private readonly service: LandingPageCmsService) {
    }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
      [
          { name: "image", maxCount: 1 },
          { name: "logo", maxCount: 1 }
      ]
    ))
    updateOrCreateData(
      @Body() dto: UpdateLandingPageCmsDto,
      @UploadedFiles() files: {
          image: UploadedMulterFileI,
          logo: UploadedMulterFileI
      }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
