import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { UploadedMulterFileI } from "../../SpacesModule/SpacesService";
import { DoSpacesService } from "../../SpacesModule/SpacesService/doSpacesService";
import { UpdateUserPageBannerCmsDto } from "./dto/update-home-page-section-seven-cms.dto";
import { UserPageBannerCms } from "./entities/user-page-banner-cms.entity";
import { UserPageBannerCmsRepository } from "./user-page-banner-cms.repository";

@Injectable()
export class UserPageBannerCmsService {
    constructor(
      @InjectRepository(UserPageBannerCmsRepository)
      private readonly repository: UserPageBannerCmsRepository,
      private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
      dto: UpdateUserPageBannerCmsDto,
      files: {
          image: UploadedMulterFileI,
      }
    ): Promise<UserPageBannerCms | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "UserPageBannerCms/");
            dto.image = image;
        }
        let isImageOrColor: any = dto.isImageOrColor;
        if (isImageOrColor === "true") {
            dto.isImageOrColor = true;
        } else {
            dto.isImageOrColor = false;
        }
        const isExists = await this.repository.find({ isDeleted: false });
        
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                  Constants.NOT_FOUND,
                  HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                  Constants.UPDATE_FAILED,
                  HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<UserPageBannerCms[] | Error> {
        const data = await this.repository.find({
            isDeleted: false
        });
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}