import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { AboutPageSectionTwoService } from "./about-page-section-two.service";
import { CreateAboutPageSectionTwoDto } from "./dto/create-about-page-section-two.dto";
import { UpdateAboutPageSectionTwoDto } from "./dto/update-about-page-section-two.dto";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "../SpacesModule/SpacesService";

@Controller({ path: "about-page-section-two", version: Constants.API_VERSION_1 })

export class AboutPageSectionTwoController {
    constructor(private readonly service: AboutPageSectionTwoService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
      [
          { name: "image", maxCount: 1 }
      ]
    ))
    create(@Body() dto: CreateAboutPageSectionTwoDto,
           @UploadedFiles() files: {
               image: UploadedMulterFileI,
           }) {
        return this.service.create(dto, files);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
      [
          { name: "image", maxCount: 1 }
      ]
    ))
    update(
      @Param("id") id: string,
      @Body() dto: UpdateAboutPageSectionTwoDto,
      @UploadedFiles() files: {
          image: UploadedMulterFileI,
      }) {
        return this.service.update(id, dto, files);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
