import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateAboutPageSectionTwoDto } from "./dto/create-about-page-section-two.dto";
import { UpdateAboutPageSectionTwoDto } from "./dto/update-about-page-section-two.dto";
import { AboutPageSectionTwo } from "./entities/about-page-section-two.entity";

@EntityRepository(AboutPageSectionTwo)
export class AboutPageSectionTwoRepository extends Repository<AboutPageSectionTwo> {
    async createEntity(dto: CreateAboutPageSectionTwoDto): Promise<AboutPageSectionTwo | Error> {
        try {
            const {
                title,
                description,
                image
            } = dto;
            const entity = this.create({
                title,
                description,
                image
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<AboutPageSectionTwo | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateAboutPageSectionTwoDto): Promise<AboutPageSectionTwo | Error> {
        try {
            await this.update(id, {
                title: dto.title,
                description: dto.description,
                image: dto.image
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<AboutPageSectionTwo[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
              .createQueryBuilder()
              .update(AboutPageSectionTwo)
              .set({ isDeleted: true })
              .where("id = :id", { id: id })
              .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}