import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { ContactReasonRepository } from "./contact-reason.repository";
import { CreateContactReasonDto } from "./dto/create-contact-reason.dto";
import { UpdateContactReasonDto } from "./dto/update-contact-reason.dto";
import { ContactReason } from "./entities/contact-reason.entity";

@Injectable()
export class ContactReasonService {
    constructor(
        @InjectRepository(ContactReasonRepository)
        private readonly repository: ContactReasonRepository
    ) {
    }

    async create(dto: CreateContactReasonDto): Promise<ContactReason | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Your message stored successfully", "data", data);
    }

    async update(id: string, dto: UpdateContactReasonDto): Promise<ContactReason | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async findAll(stateCode: string): Promise<ContactReason[] | Error> {
        const data = await this.repository.findByFilterQuery({stateCode: stateCode});
        if (!data) {
        throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findAllContactReason(stateCode: string): Promise<ContactReason[] | Error> {
        const data = await this.repository.findByFilterQuery({stateCode: stateCode, isShow: true });
        if (!data) {
        throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<ContactReason[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }

    async makeDefault(dto: any): Promise<any | Error> {
        // Retrieve all entities
        const entities = await this.repository.find();

        // Update the isDefault property for each entity
        for (const entity of entities) {
            const updateData = await this.repository.update(entity.id, { isDefault: false });
        }
        ;

        // retrive data using state code
        const data = await this.repository.find({
            stateCode: dto.stateCode,
            isDeleted: false
        });

        for (const entity of data) {
            const updateData = await this.repository.update(entity.id, { isDefault: true });
        }
        ;

        const finalData = await this.repository.find({
            stateCode: dto.stateCode,
            isDeleted: false
        });

        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", finalData);
    }

    async isShow(id: string, dto: any): Promise<any|Error> {
        const data = await this.repository.findOne({
            id: id
        });
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        const updateData = await this.repository.update(id, {
            isShow: dto.isShow
        });

        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }
}
