import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { getGraphQLClient } from "src/helper/graphql-client-helper";
import {
    fetchAllProduct,
    fetchAllProductBrand,
    fetchAllProductCategory,
    fetchAllProductEffects,
    fetchAllProductStrainType,
    fetchOneProduct,
    fetchProductsCount,
    filterListQuery,
    filterProducts,
    menuType
} from "src/helper/dutchie-plus-query";
import { DutchiePlusRetailerService } from "../dutchie-plus-retailer/dutchie-plus-retailer.service";
import { InjectRepository } from "@nestjs/typeorm";
import {
    ProductCategoryRepository
} from "src/modules/dutchie-plus/custom-services/product-category/product-category.repository";
import { ProductEffectsRepository } from "../custom-services/product-effects/product-effects.repository";
import { ProductServiceRepository } from "../custom-services/product-service/product-service.repository";
import { Constants } from "src/utils/constants";
import { StateWiseProductCategoryRepository } from "../custom-services/state-wise-product-category/state-wise-product-category.repository";


@Injectable()
export class DutchiePlusProductService {
    constructor(
      private readonly retailerService: DutchiePlusRetailerService,
      @InjectRepository(ProductCategoryRepository)
      private readonly productCatRepo: ProductCategoryRepository,
      @InjectRepository(ProductEffectsRepository)
      private readonly productEffectRepo: ProductEffectsRepository,
      @InjectRepository(ProductServiceRepository)
      private readonly productServiceRepo: ProductServiceRepository,
      private readonly stateWiseCaregoryRepo: StateWiseProductCategoryRepository
    ) {
    }

    async findAll(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchAllProduct();
            return await graphQLClient.request(query, variables);
        } catch (e) {
            return e;
        }
    }

    async findOne(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchOneProduct();
            return await graphQLClient.request(query, variables);
        } catch (e) {
            return e;
        }
    }

    async findAllBrands(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchAllProductBrand();
            const data = await graphQLClient.request(query, variables);
            return data["menu"];
        } catch (e) {
            return e;
        }
    }

    async filterProducts(
      variables: any
    ) {
        try {
            const graphQLClient = getGraphQLClient();
            const filterProductsQuery = filterProducts();
            const data = await graphQLClient.request(filterProductsQuery, variables);

            let subCategory = [];
            if (variables.filter.category && data.menu.products.length > 0) {
                subCategory = [...new Set(data.menu.products.map(item => item.subcategory))];
            }

            if (data.menu && typeof variables.maxPrice === "undefined" && typeof variables.minPrice === "undefined") {
                data.menu.subCategory = subCategory;
            }

            const productCount = data.menu.productsCount;

            if (typeof variables.maxPrice !== "undefined" && typeof variables.minPrice !== "undefined") {
                const minPrice = variables.minPrice;
                const maxPrice = variables.maxPrice;
                const variableObject = {
                    retailerId: variables.retailerId,
                    filter: variables.filter,
                    pagination: {
                        offset: 0,
                        limit: productCount
                    }
                };

                if (typeof variables.menuType !== "undefined") {
                    variableObject["menuType"] = variables.menuType;
                }

                const filterProductsQuery = filterProducts();
                const filteredData = await graphQLClient.request(filterProductsQuery, variableObject);

                const filteredProducts = filteredData.menu.products.filter(product => {
                    const price = product.variants[0].priceMed;
                    return price >= minPrice && price <= maxPrice;
                });

                data.menu.productsCount = filteredProducts.length;
                data.menu.products = filteredProducts.splice(variables.pagination.offset, variables.pagination.limit);

                if (data.menu) {
                    data.menu.subCategory = subCategory;
                }

                return data;
            } else {
                return data;
            }
        } catch (e) {
            return e;
        }

    }

    async findAllCategory(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchAllProductCategory();
            const data = await graphQLClient.request(query, variables);
            return data["menu"];
        } catch (e) {
            return e;
        }
    }

    async findUniqueCategory(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();

            const productCountQuery = fetchProductsCount();
            const countData = await graphQLClient.request(productCountQuery, variables);
            const categoryVariable = {
                retailerId: variables.retailerId,
                pagination: {
                    offset: 0,
                    limit: countData.menu.productsCount
                }
            };
            const fetchAllProductCategoryQuery = filterListQuery();
            const data = await graphQLClient.request(fetchAllProductCategoryQuery, categoryVariable);

            const uniqueCategories = [...new Set(data.menu.products.map(item => item.category))];

            const productCategoryPromises = uniqueCategories.map(async (category) => {
                // // const productCategory = await this.productCatRepo.findOne({
                // //     where: {
                // //         categoryName: category,
                // //         isDeleted: false
                // //     }
                // // });

                // if (productCategory) {
                //     return {
                //         category: productCategory.categoryName,
                //         image: productCategory.image,
                //         featuredImage: productCategory.featuredImage
                //     };
                // }
            });

            const productUniqueCategoryList = await Promise.all(productCategoryPromises);

            const effectsData = data.menu.products;
            const uniqueEffects = [];

            for (const product of effectsData) {
                for (const effect of product.effects) {
                    if (!uniqueEffects.includes(effect)) {
                        uniqueEffects.push(effect);
                    }
                }
            }

            const payload = {
                productUniqueCategoryList: productUniqueCategoryList,
                uniqueEffects: uniqueEffects
            };

            return payload;
        } catch (e) {
            return e;
        }
    }

    async findAllStrainType(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchAllProductStrainType();
            const data = await graphQLClient.request(query, variables);
            return data["menu"];
        } catch (e) {
            return e;
        }
    }

    async findAllEffects(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchAllProductEffects();
            const data = await graphQLClient.request(query, variables);
            return data["menu"];
        } catch (e) {
            return e;
        }
    }

    async findAllUniqueEffects(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const fetchAllProductEffectsQuery = fetchAllProductEffects();
            const data = await graphQLClient.request(fetchAllProductEffectsQuery, variables);

            const effectsData = data.menu.products;
            const uniqueEffects = [];

            for (const product of effectsData) {
                for (const effect of product.effects) {
                    if (!uniqueEffects.includes(effect)) {
                        uniqueEffects.push(effect);
                    }
                }
            }

            return uniqueEffects;
        } catch (e) {
            return e;
        }
    }


    async filterList(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const productCountQuery = fetchProductsCount();
            const countData = await graphQLClient.request(productCountQuery, variables);
            const filterVariable = {
                retailerId: variables.retailerId,
                pagination: {
                    offset: 0,
                    limit: countData.menu.productsCount
                }
            };

            const filterListQ = filterProducts();
            const filterListResponse = await graphQLClient.request(filterListQ, filterVariable);
            const filterList = filterListResponse.menu.products;

            // Find all unique categories
            const categories = [...new Set(filterList.map(item => item.category))];

            // Find all unique subcategories
            const subCategories = [...new Set(filterList.map(item => item.subcategory))];

            // Find all unique strain types
            const strainTypes = [...new Set(filterList.map(item => item.strainType))];

            // Find all unique effects
            const effects = new Set();
            for (const filter of filterList) {
                for (const effect of filter.effects) {
                    effects.add(effect);
                }
            }
            const uniqueEffects = Array.from(effects);

            // Get the weights
            const weights = filterListResponse.menu.weights;

            // Get the brands
            const brands = filterListResponse.menu.brands.map(item => {
                return {
                    id: item.id,
                    name: item.name
                };
            });

            // Get the menu types for the retailer
            const menuTypeQuery = menuType();
            const menuTypeResponse = await graphQLClient.request(menuTypeQuery, { id: variables.retailerId });
            const menuTypeList = menuTypeResponse.retailer ? menuTypeResponse.retailer.menuTypes : [];


            if (typeof variables.price !== "undefined") {
                const minPrice = 0;
                const maxPrice = variables.price;

                const filteredProducts = filterListResponse.menu.products.filter(product => {
                    const price = product.variants[0].priceMed;
                    return price >= minPrice && price <= maxPrice;
                });

                filterListResponse.menu.productsCount = filteredProducts.length;
                filterListResponse.menu.products = filteredProducts.splice(0, 60);

                return {
                    categories,
                    subCategories,
                    strainTypes,
                    effects: uniqueEffects,
                    weights,
                    brands,
                    menuTypes: menuTypeList,
                    products: filterListResponse
                };
            } else {
                return {
                    categories,
                    subCategories,
                    strainTypes,
                    effects: uniqueEffects,
                    weights,
                    brands,
                    menuTypes: menuTypeList
                };
            }
        } catch (e) {
            return e;
        }
    }


    async customFilterList(stateCode: string): Promise<any | Error> {
        let finalCategory = []
        const data: any = await this.stateWiseCaregoryRepo.find({
            stateCode: stateCode,
            isShown: true,
            isDeleted: false
        });
        if (Object.keys(data).length === 0) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        for(let i=0; i< Object.keys(data).length; i++) {
            const productCategory: any = await this.productCatRepo.findOne({
                id: data[i].categoryId,
                isDeleted: false
            });
            if(productCategory){
                productCategory.position = data[i].position;
                productCategory.serial = data[i].serial;
                finalCategory.push(productCategory);
            }
        }
        // const uniqueCategoryWithSerial = data.map((item, index) => ({
        //     ...item,
        //     serial: index + 1
        // }));
        // const orderedBrands = uniqueBrandsWithSerial.sort((a, b) => a.position - b.position);
        const orderedCategory = finalCategory.sort((a, b) => {
            if (a.position && b.position) {
                return a.position - b.position;
            } else if (a.position) {
                return -1; // Place objects with position first
            } else if (b.position) {
                return 1; // Place objects with position last
            }
            return a.serial - b.serial; // If no position, sort by serial
        });

        const effects = await this.productEffectRepo.find({
            order: { effects: "ASC" },
            where: {
                isDeleted: false
            }
        });

        const strainType = await this.productServiceRepo.getProductStrainType();


        return {
            category: orderedCategory.map(item => item.categoryName),
            effects: effects.map(item => item.effects),
            strainType: strainType
              .filter(item => item.strainType !== null)
              .map(item => item.strainType)
        };
    }
}
