import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    Query,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "../../../SpacesModule/SpacesService";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { SearchRetailerIHeartJaneDto } from "./dto/search-retailer-iheartjnae.dto";
import { RetailerServiceIHeartJaneService } from "./retailer-service-iheartjane.service";
import { CreateRetailerServiceIHeartJaneDto } from "./dto/create-retailer-service-iheartjane.dto";

@Controller({ path: "retailer-service-iheartjane", version: Constants.API_VERSION_1 })

export class RetailerServiceIHeartJaneController {
    constructor(private readonly service: RetailerServiceIHeartJaneService) {
    }

    @Post()
    // @UseGuards(AdminAuthGuard)
    create(
        @Body() dto: CreateRetailerServiceIHeartJaneDto
    ) {
        return this.service.create(dto);
    }

    @Get("/synchronize")
    // @UseGuards(AdminAuthGuard)
    synchronizeRetailers() {
        return this.service.createUpdateRetailer();
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    // @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/alpineiq-update/:id")
    updateAlpineIq(@Param("id") id: string, @Body() dto: any) {
        return this.service.updateAlpineIq(id, dto);
    }


    @Patch(":id")
    // @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: "retailerImage" },
            { name: "featuredImage", maxCount: 1 }
        ]
    ))
    update(
        @Param("id") id: string,
        @Body() dto: any,
        @UploadedFiles() files: {
            retailerImage?: UploadedMulterFileI,
            featuredImage?: UploadedMulterFileI
        }
    ) {
        return this.service.update(id, dto, files);
    }

    @Delete(":id")
    // @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }

    @Get("/populate/elasticsearch")
    @UseGuards(AdminAuthGuard)
    async productPopulateElasticsearch() {
        let index: string;
        if (process.env.ENVIRONMENT == "local") {
            index = process.env.RETAILER_INDEX_DEV;
        } else {
            index = process.env.RETAILER_INDEX_PROD;
        }
        return await this.service.indexBulkData(index);
    }

    @Post("/elastic/search")
    async productElasticsearch(@Body() searchDto: SearchRetailerIHeartJaneDto) {
        let index: string;
        if (process.env.ENVIRONMENT == "local") {
            index = process.env.RETAILER_INDEX_DEV;
        } else {
            index = process.env.RETAILER_INDEX_PROD;
        }
        return await this.service.search(index, searchDto);
    }

    
    @Get("/stores/closest-retailers")
    // @UseGuards(AdminAuthGuard)
    findClosestRetailer(
        @Query("lat") lat: string,
        @Query("long") long: string ) {
        return this.service.findClosestRetailer(lat, long);
    }
}
