import {
    Body,
    Controller,
    Get,
    HttpCode,
    HttpStatus,
    Post,
    Query,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { Constants } from "src/utils/constants";
import { ShopDealsPageCmsService } from "./shop-deals-page-cms.service";
import { UpdateShopDealsPageCmsDto } from "./dto/update-shop-deals-page-cms.dto";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";

@Controller({ path: "shop-deals-page-cms", version: Constants.API_VERSION_1 })

export class ShopDealsPageCmsController {
    constructor(private readonly service: ShopDealsPageCmsService) {
    }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
      [
          { name: "shopDealSectionOneBackgroundImage", maxCount: 1 },
            { name: "shopDealBannerOneImage", maxCount: 1 },
            { name: "shopDealBannerTwoDesktopImage", maxCount: 1 },
            { name: "shopDealBannerTwoMobileImage", maxCount: 1 },
            { name: "shopDealBannerThreeImage", maxCount: 1 },
            { name: "shopDealFirstPurchaseBackgoundImage", maxCount: 1 },
      ]
    ))
    updateOrCreateData(
      @Body() dto: UpdateShopDealsPageCmsDto,
      @UploadedFiles() files: {
          shopDealSectionOneBackgroundImage: UploadedMulterFileI,
          shopDealBannerOneImage: UploadedMulterFileI,
          shopDealBannerTwoDesktopImage: UploadedMulterFileI,
          shopDealBannerTwoMobileImage: UploadedMulterFileI,
          shopDealBannerThreeImage: UploadedMulterFileI,
          shopDealFirstPurchaseBackgoundImage: UploadedMulterFileI,
      }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
