import { EntityRepository, getConnection, Repository } from "typeorm";
import { UpdateWeeklyAdDto } from "./dto/update-weekly-ad.dto";
import { WeeklyAdEntity } from "./entities/weekly-ad.entity";
import { BaseWeeklyAdDto } from "./dto/base-weekly-ad.dto";

@EntityRepository(WeeklyAdEntity)
export class WeeklyAdRepository extends Repository<WeeklyAdEntity> {

    async createEntity(dto: BaseWeeklyAdDto): Promise<WeeklyAdEntity | Error> {
        try {
            const entity = this.create({
                stateCode: dto.stateCode,
                stateName: dto.stateName,
                title: dto.title,
                buttonText: dto.buttonText,
                buttonLink: dto.buttonLink,
                backgroundImage: dto.backgroundImage
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(dto: UpdateWeeklyAdDto, id: string): Promise<WeeklyAdEntity | Error> {
        try {
            await this.update(id, {
                stateCode: dto.stateCode,
                stateName: dto.stateName,
                title: dto.title,
                buttonText: dto.buttonText,
                buttonLink: dto.buttonLink,
                backgroundImage: dto.backgroundImage
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<WeeklyAdEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<WeeklyAdEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<WeeklyAdEntity[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
              .createQueryBuilder()
              .update(WeeklyAdEntity)
              .set({ isDeleted: true })
              .where("id = :id", { id: id })
              .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}